(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('rxjs/add/observable/interval')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common', 'rxjs/add/observable/interval'], factory) :
	(factory((global['angular-drag-bounce'] = {}),global.core,global.common));
}(this, (function (exports,core,common) { 'use strict';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var BOUNCEABLE_CFG = new core.InjectionToken('as.bounceable.config');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var Vector = (function () {
    function Vector(x, y) {
        if (x === void 0) { x = 0; }
        if (y === void 0) { y = 0; }
        this.x = x;
        this.y = y;
    }
    Object.defineProperty(Vector.prototype, "length", {
        get: /**
         * @return {?}
         */
        function () {
            return Math.sqrt((this.x * this.x) + (this.y * this.y));
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} x
     * @param {?} y
     * @return {?}
     */
    Vector.prototype.set = /**
     * @param {?} x
     * @param {?} y
     * @return {?}
     */
    function (x, y) {
        this.x = x;
        this.y = y;
    };
    /**
     * @return {?}
     */
    Vector.prototype.normalize = /**
     * @return {?}
     */
    function () {
        this.multiply(1 / this.length);
    };
    /**
     * @param {?} x
     * @param {?} y
     * @return {?}
     */
    Vector.prototype.add = /**
     * @param {?} x
     * @param {?} y
     * @return {?}
     */
    function (x, y) {
        this.x += x;
        this.y += y;
    };
    /**
     * @param {?} x
     * @param {?} y
     * @return {?}
     */
    Vector.prototype.subtract = /**
     * @param {?} x
     * @param {?} y
     * @return {?}
     */
    function (x, y) {
        this.add(-x, -y);
    };
    /**
     * @param {?} factor
     * @return {?}
     */
    Vector.prototype.multiply = /**
     * @param {?} factor
     * @return {?}
     */
    function (factor) {
        this.x *= factor;
        this.y *= factor;
    };
    /**
     * @param {?} factor
     * @return {?}
     */
    Vector.prototype.divide = /**
     * @param {?} factor
     * @return {?}
     */
    function (factor) {
        this.multiply(1 / factor);
    };
    return Vector;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var BounceableService = (function () {
    function BounceableService(_bounceableConfig) {
        var _this = this;
        this._bounceableConfig = _bounceableConfig;
        this._items = [];
        this._dragStart = new Vector();
        this.onMouseDown = function ($event) {
            _this._dragStart = new Vector($event.clientX, $event.clientY);
            _this._items.forEach(function (i) { return i.onMouseDown($event); });
        };
        this.onMouseUp = function ($event) {
            var /** @type {?} */ momentum = new Vector(($event.clientX - _this._dragStart.x) * _this._bounceableConfig.momentumSlowDownFactor, ($event.clientY - _this._dragStart.y) * _this._bounceableConfig.momentumSlowDownFactor);
            _this._items.forEach(function (i) { return i.onMouseUp(momentum); });
            _this.forceUpdate();
        };
        this._timeUntilNextRenderMS = 1000 / this._bounceableConfig.framesPerSecond;
        this.setupListeners();
    }
    /**
     * @param {?} item
     * @return {?}
     */
    BounceableService.prototype.register = /**
     * @param {?} item
     * @return {?}
     */
    function (item) {
        this._items.push(item);
        this.forceUpdate();
    };
    /**
     * @param {?} item
     * @return {?}
     */
    BounceableService.prototype.unregister = /**
     * @param {?} item
     * @return {?}
     */
    function (item) {
        this._items = this._items.filter(function (i) { return i !== item; });
    };
    /**
     * @return {?}
     */
    BounceableService.prototype.getItems = /**
     * @return {?}
     */
    function () {
        return this._items;
    };
    /**
     * @return {?}
     */
    BounceableService.prototype.step = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.clearTimeout();
        this._items.forEach(function (item) { return item.step(); });
        if (this.itemsAreStillMoving()) {
            this._nextRenderTimeout = setTimeout(function () { return _this.step(); }, this._timeUntilNextRenderMS);
        }
    };
    /**
     * @return {?}
     */
    BounceableService.prototype.forceUpdate = /**
     * @return {?}
     */
    function () {
        if (!this._nextRenderTimeout) {
            this.step();
        }
    };
    /**
     * @return {?}
     */
    BounceableService.prototype.clearTimeout = /**
     * @return {?}
     */
    function () {
        clearTimeout(this._nextRenderTimeout);
        this._nextRenderTimeout = null;
    };
    /**
     * @return {?}
     */
    BounceableService.prototype.setupListeners = /**
     * @return {?}
     */
    function () {
        document.addEventListener('mousedown', this.onMouseDown);
        document.addEventListener('mouseup', this.onMouseUp);
    };
    /**
     * @return {?}
     */
    BounceableService.prototype.itemsAreStillMoving = /**
     * @return {?}
     */
    function () {
        return this._items.some(function (i) { return i.isMoving; });
    };
    BounceableService.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    BounceableService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: core.Inject, args: [BOUNCEABLE_CFG,] },] },
    ]; };
    return BounceableService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var CollisionService = (function () {
    function CollisionService() {
    }
    /**
     * @param {?} item1
     * @param {?} item2
     * @return {?}
     */
    CollisionService.prototype.doItemsCollide = /**
     * @param {?} item1
     * @param {?} item2
     * @return {?}
     */
    function (item1, item2) {
        return item1.left < item2.right
            && item1.right > item2.left
            && item1.top < item2.bottom
            && item1.bottom > item2.top;
    };
    /**
     * @param {?} item1
     * @param {?} item2
     * @return {?}
     */
    CollisionService.prototype.getItemsOverlap = /**
     * @param {?} item1
     * @param {?} item2
     * @return {?}
     */
    function (item1, item2) {
        var /** @type {?} */ overlapXside1 = Math.abs(item1.left - item2.right);
        var /** @type {?} */ overlapXside2 = Math.abs(item1.right - item2.left);
        var /** @type {?} */ overlapYside1 = Math.abs(item1.top - item2.bottom);
        var /** @type {?} */ overlapYside2 = Math.abs(item1.bottom - item2.top);
        return new Vector(Math.min(overlapXside1, overlapXside2), Math.min(overlapYside1, overlapYside2));
    };
    CollisionService.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    CollisionService.ctorParameters = function () { return []; };
    return CollisionService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var BounceableComponent = (function () {
    function BounceableComponent(_bounceableConfig, _bounceableService, _changeDetectorRef, _collisionService, _elementRef) {
        this._bounceableConfig = _bounceableConfig;
        this._bounceableService = _bounceableService;
        this._changeDetectorRef = _changeDetectorRef;
        this._collisionService = _collisionService;
        this._elementRef = _elementRef;
        this.position = new Vector();
        this.momentum = new Vector();
    }
    /**
     * @return {?}
     */
    BounceableComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this._bounceableService.register(this);
        this._changeDetectorRef.detach();
    };
    /**
     * @param {?} changes
     * @return {?}
     */
    BounceableComponent.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        var position = changes.position, momentum = changes.momentum;
        if (position && position.currentValue) {
            this.position = new Vector(position.currentValue.x, position.currentValue.y);
        }
        if (momentum && momentum.currentValue) {
            this.momentum = new Vector(momentum.currentValue.x, momentum.currentValue.y);
        }
    };
    /**
     * @return {?}
     */
    BounceableComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        this.width = this._elementRef.nativeElement.offsetWidth;
        this.height = this._elementRef.nativeElement.offsetHeight;
        this.weight = this.width * this.height;
        this.updatePosition();
    };
    /**
     * @return {?}
     */
    BounceableComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this._bounceableService.unregister(this);
    };
    /**
     * @param {?} $event
     * @return {?}
     */
    BounceableComponent.prototype.onMouseDown = /**
     * @param {?} $event
     * @return {?}
     */
    function ($event) {
        this.isDragging = false;
        if (this._elementRef.nativeElement.contains($event.target)) {
            $event.preventDefault();
            this.isDragging = true;
        }
    };
    /**
     * @param {?} momentum
     * @return {?}
     */
    BounceableComponent.prototype.onMouseUp = /**
     * @param {?} momentum
     * @return {?}
     */
    function (momentum) {
        if (!this.isDragging) {
            return;
        }
        this.momentum = new Vector(momentum.x, momentum.y);
        this.isMoving = true;
        this.isDragging = false;
    };
    /**
     * @return {?}
     */
    BounceableComponent.prototype.step = /**
     * @return {?}
     */
    function () {
        if (!this.isMoving) {
            return;
        }
        this.handleCollisions();
        this.handleOutOfContainerBounds();
        this.applyAirFriction();
        this.updatePosition();
    };
    Object.defineProperty(BounceableComponent.prototype, "bottom", {
        get: /**
         * @return {?}
         */
        function () {
            return this.position.y + this.height;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BounceableComponent.prototype, "top", {
        get: /**
         * @return {?}
         */
        function () {
            return this.position.y;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BounceableComponent.prototype, "right", {
        get: /**
         * @return {?}
         */
        function () {
            return this.position.x + this.width;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BounceableComponent.prototype, "left", {
        get: /**
         * @return {?}
         */
        function () {
            return this.position.x;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    BounceableComponent.prototype.handleCollisions = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this._bounceableService.getItems()
            .filter(function (item) { return item !== _this; })
            .filter(function (item) { return _this._collisionService.doItemsCollide(item, _this); })
            .forEach(function (item) { return _this.respondToCollisionWithItem(item); });
    };
    /**
     * @return {?}
     */
    BounceableComponent.prototype.handleOutOfContainerBounds = /**
     * @return {?}
     */
    function () {
        var /** @type {?} */ left = this.position.x;
        var /** @type {?} */ right = this.position.x + this.width;
        var /** @type {?} */ top = this.position.y;
        var /** @type {?} */ bottom = this.position.y + this.height;
        if (left < 0) {
            this.position.x = 0;
            this.momentum.x *= -this._bounceableConfig.edgeBounceFrictionFactor;
        }
        if (right > document.body.clientWidth) {
            this.position.x = document.body.clientWidth - this.width;
            this.momentum.x *= -this._bounceableConfig.edgeBounceFrictionFactor;
        }
        if (top < 0) {
            this.position.y = 0;
            this.momentum.y *= -this._bounceableConfig.edgeBounceFrictionFactor;
        }
        if (bottom > document.body.clientHeight) {
            this.position.y = document.body.clientHeight - this.height;
            this.momentum.y *= -this._bounceableConfig.edgeBounceFrictionFactor;
        }
    };
    /**
     * @param {?} item
     * @return {?}
     */
    BounceableComponent.prototype.respondToCollisionWithItem = /**
     * @param {?} item
     * @return {?}
     */
    function (item) {
        var /** @type {?} */ overlapVector = this._collisionService.getItemsOverlap(this, item);
        var /** @type {?} */ overlapRatioX = Math.abs(overlapVector.x / this.momentum.x);
        var /** @type {?} */ overlapRatioY = Math.abs(overlapVector.y / this.momentum.y);
        var /** @type {?} */ itemWeightRatio = this.weight / (this.weight + item.weight);
        var /** @type {?} */ thisWeightRatio = 1 - itemWeightRatio;
        var /** @type {?} */ totalImpulse = new Vector(this.momentum.x + item.momentum.x, this.momentum.y + item.momentum.y);
        var /** @type {?} */ itemImpulse = new Vector(totalImpulse.x * itemWeightRatio, totalImpulse.y * itemWeightRatio);
        var /** @type {?} */ thisImpulse = new Vector(totalImpulse.x * thisWeightRatio, totalImpulse.y * thisWeightRatio);
        var /** @type {?} */ setBackVector = new Vector(this.momentum.x, this.momentum.y);
        setBackVector.multiply((Math.abs(overlapRatioX) < Math.abs(overlapRatioY) ? overlapRatioX : overlapRatioY) * -1.001);
        this.position.add(setBackVector.x, setBackVector.y);
        this.momentum.set(-thisImpulse.x, -thisImpulse.y);
        item.momentum.add(itemImpulse.x, itemImpulse.y);
        item.isMoving = true;
    };
    /**
     * @return {?}
     */
    BounceableComponent.prototype.applyAirFriction = /**
     * @return {?}
     */
    function () {
        this.momentum.multiply(this._bounceableConfig.airFrictionFactor);
        if (Math.abs(this.momentum.x) < this._bounceableConfig.momentumNullThreshold) {
            this.momentum.x = 0;
        }
        if (Math.abs(this.momentum.y) < this._bounceableConfig.momentumNullThreshold) {
            this.momentum.y = 0;
        }
        this.isMoving = !(this.momentum.x === 0 && this.momentum.y === 0);
    };
    /**
     * @return {?}
     */
    BounceableComponent.prototype.updatePosition = /**
     * @return {?}
     */
    function () {
        this.position.add(this.momentum.x, this.momentum.y);
        this._elementRef.nativeElement.style.transform = "translate3d(" + this.position.x + "px, " + this.position.y + "px, 0px)";
    };
    BounceableComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'as-bounceable',
                    template: '<ng-content></ng-content>',
                    styles: ["as-bounceable { display: inline-block; cursor: pointer; position: fixed; top: 0px; left: 0px; } as-bounceable * { -webkit-user-select: none; -moz-user-select: none; user-select: none; } "],
                    encapsulation: core.ViewEncapsulation.None
                },] },
    ];
    /** @nocollapse */
    BounceableComponent.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: core.Inject, args: [BOUNCEABLE_CFG,] },] },
        { type: BounceableService, },
        { type: core.ChangeDetectorRef, },
        { type: CollisionService, },
        { type: core.ElementRef, },
    ]; };
    BounceableComponent.propDecorators = {
        "position": [{ type: core.Input },],
        "momentum": [{ type: core.Input },],
    };
    return BounceableComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

var DEFAULT_CONFIG = {
    framesPerSecond: 50,
    momentumSlowDownFactor: 0.1,
    momentumNullThreshold: 0.5,
    airFrictionFactor: 0.9,
    edgeBounceFrictionFactor: 0.5
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var BounceableModule = (function () {
    function BounceableModule() {
    }
    /**
     * @param {?=} config
     * @return {?}
     */
    BounceableModule.initialize = /**
     * @param {?=} config
     * @return {?}
     */
    function (config) {
        return {
            ngModule: BounceableModule,
            providers: [{
                    provide: BOUNCEABLE_CFG,
                    useValue: Object.assign(Object.assign({}, DEFAULT_CONFIG), config || {})
                }]
        };
    };
    BounceableModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [common.CommonModule],
                    providers: [BounceableService, CollisionService],
                    declarations: [BounceableComponent],
                    exports: [BounceableComponent]
                },] },
    ];
    /** @nocollapse */
    BounceableModule.ctorParameters = function () { return []; };
    return BounceableModule;
}());

exports.BounceableModule = BounceableModule;
exports.BounceableComponent = BounceableComponent;
exports.DEFAULT_CONFIG = DEFAULT_CONFIG;
exports.BOUNCEABLE_CFG = BOUNCEABLE_CFG;
exports.BounceableService = BounceableService;
exports.CollisionService = CollisionService;
exports.Vector = Vector;

Object.defineProperty(exports, '__esModule', { value: true });

})));
